/***************************************************************************
 *   Copyright (C) 2004 by David Sansome                                   *
 *   me@davidsansome.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../config.h"
#ifdef HAVE_AUTOPACKAGE

#include <unistd.h> 
#include <sys/types.h> 
#include <sys/wait.h> 
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <locale.h>

#include "core.h"

char* globalDbLocation;
char* userDbLocation;
char* locale;
char* locale1;
char* locale2;

void autopackageUninstall(char* packageName, void (*callback)(int))
{
	char* commandPrefix = "bash -c \"[ -e /etc/autopackage/config ] && source /etc/autopackage/config; [ -e ~/.config/autopackage/config ] && source ~/.config/autopackage/config; source \\\"\\$autopackage_prefix/share/autopackage/apkg-funclib\\\"; _initializeAutopackage; uninstallPackage \\\"";
	
	char* command = (char*) malloc(strlen(commandPrefix) + strlen(packageName) + 4);
	strcpy(command, commandPrefix);
	strcat(command, packageName);
	strcat(command, "\\\"\"");
	
	int ret = system(command);
	
	callback(WEXITSTATUS(ret));
}

/* This function executes the command given in the argument, and returns its stdout */
char* runCommand(char* command)
{
	FILE* p = popen(command, "r");
	if ((p == NULL) || (p < 0))
		return NULL;
	
	char* output = NULL;
	int len = 0;
	while (!feof(p))
	{
		char buffer[256];
		int n = fread(buffer, 1, 256, p);
		output = (char*)realloc(output, len + n);
		memcpy(output + len, &buffer, n);
		len += n;
	}
	pclose(p);
	
	output = (char*)realloc(output, len + 1);
	output[len] = '\0';
	
	return output;
}

// Returns 0 for success, 1 for failure
int parseDebianFile(const char* path, int root, const char* shortname, int size, char* timestamp, void (*callback)(struct ApplicationInfo*))
{
	FILE* desktopFileHandle = fopen(path, "r");
	if (desktopFileHandle == NULL)
		return 1;
	
	char* desktopFile = NULL;
	int len = 0;
	while (!feof(desktopFileHandle))
	{
		char buffer[256];
		int n = fread(buffer, 1, 256, desktopFileHandle);
		desktopFile = (char*)realloc(desktopFile, len + n);
		memcpy(desktopFile + len, &buffer, n);
		len += n;
	}
	fclose(desktopFileHandle);
	
	char* startPos = strstr(desktopFile, "icon=\"");
	if (startPos == NULL)
		return 1;
	startPos += 6;
	char* endPos = strstr(startPos, "\"");
	if (endPos == NULL)
		return 1;
	char* icon = (char*) malloc(endPos - startPos + 1);
	memcpy(icon, startPos, endPos - startPos);
	icon[endPos - startPos] = '\0';
	
	
	startPos = strstr(desktopFile, "title=\"");
	if (startPos == NULL)
		return 1;
	startPos += 7;
	endPos = strstr(startPos, "\"");
	if (endPos == NULL)
		return 1;
	char* title = (char*) malloc(endPos - startPos + 1);
	memcpy(title, startPos, endPos - startPos);
	title[endPos - startPos] = '\0';
	
	char* shortnameCopy = (char*) malloc(strlen(shortname) + 1);
	strcpy(shortnameCopy, shortname);
	
	char* timestampCopy = (char*) malloc(strlen(timestamp) + 1);
	strcpy(timestampCopy, timestamp);
	
	callback(createAppInfo(autopackageUninstall, title, icon, root, shortnameCopy, size, timestampCopy));
	
	return 0;
}

// Retunrs 
int parseDesktop(const char* path, const char* shortname, int size, char* timestamp, void (*callback)(struct ApplicationInfo*))
{
	FILE* desktopFileHandle = fopen(path, "r");
	if (desktopFileHandle == NULL)
		return 1;
	
	char* desktopFile = NULL;
	int len = 0;
	while (!feof(desktopFileHandle))
	{
		char buffer[256];
		int n = fread(buffer, 1, 256, desktopFileHandle);
		desktopFile = (char*)realloc(desktopFile, len + n);
		memcpy(desktopFile + len, &buffer, n);
		len += n;
	}
	fclose(desktopFileHandle);
	
	char* key = NULL;
	char* value = NULL;
	char* name = NULL;
	char* icon = NULL;
	int lastMarker = 0;
	int i;
	for (i=0 ; i<len ; i++)
	{
		if ((desktopFile[i] == '=') && (key == NULL))
		{
			key = (char*)malloc(i - lastMarker + 1);
			memcpy(key, desktopFile + lastMarker, i - lastMarker);
			key[i - lastMarker] = '\0';
			lastMarker = i + 1;
		}
		if (((desktopFile[i] == '\n') || (i == (len-1))) && (value == NULL))
		{
			value = (char*)malloc(i - lastMarker + 1);
			memcpy(value, desktopFile + lastMarker, i - lastMarker);
			value[i - lastMarker] = '\0';
			lastMarker = i + 1;
			
			if (key != NULL)
			{
				// If it's not an application, skip the whole file
				if ((strcasecmp(key, "type") == 0) && (strcasecmp(value, "application") != 0))
				{
					free(key);
					free(value);
					free(name);
					free(icon);
					return 1;
				}
				if ((strcasecmp(key, "name") == 0) && (name == NULL))
				{
					name = (char*) malloc(strlen(value) + 1);
					strcpy(name, value);
				}
				if ((strcasecmp(key, "icon") == 0) && (icon == NULL))
				{
					icon = (char*) malloc(strlen(value) + 1);
					strcpy(icon, value);
				}
			}
			
			free(key); key = NULL;
			free(value); value = NULL;
		}
	}
	
	// Try opening it for writing so we can see if we need to be root to remove it
	int root = 0;
	desktopFileHandle = fopen(path, "r+");
	if (desktopFileHandle == NULL)
		root = 1;
	else
		fclose(desktopFileHandle);
	
	char* shortnameCopy = (char*) malloc(strlen(shortname) + 1);
	strcpy(shortnameCopy, shortname);
	
	char* timestampCopy = (char*) malloc(strlen(timestamp) + 1);
	strcpy(timestampCopy, timestamp);
	
	callback(createAppInfo(autopackageUninstall, name, icon, root, shortnameCopy, size, timestampCopy));
	
	return 0;
}

void readFileList(const char* path, const char* shortname, int size, char* timestamp, void (*callback)(struct ApplicationInfo*))
{
	char* fileListPath = (char*) malloc(strlen(path) + 7);
	strcpy(fileListPath, path);
	strcat(fileListPath, "/files");
	
	FILE* fileListHandle = fopen(fileListPath, "r");
	char* fileList = NULL;
	int len = 0;
	while (!feof(fileListHandle))
	{
		char buffer[256];
		int n = fread(buffer, 1, 256, fileListHandle);
		fileList = (char*)realloc(fileList, len + n);
		memcpy(fileList + len, &buffer, n);
		len += n;
	}
	fclose(fileListHandle);
	free(fileListPath);
	
	char* debianUser = (char*) malloc(strlen(getenv("HOME")) + 7);
	strcpy(debianUser, getenv("HOME"));
	strcat(debianUser, "/.menu");
	
	char* line;
	int lastMarker = 0;
	int i;
	for (i=0 ; i<len ; i++)
	{
		if ((fileList[i] == '\n') || (i == (len-1)))
		{
			line = (char*)malloc(i - lastMarker + 1);
			memcpy(line, fileList + lastMarker, i - lastMarker);
			line[i - lastMarker] = '\0';
			lastMarker = i + 1;
			if ((strstr(line, ".desktop") != NULL) && ((strstr(line, "applnk") != NULL) || (strstr(line, "applications") != NULL)))
			{
				if (parseDesktop(line, shortname, size, timestamp, callback) == 0)
					break;
			}
			if (strstr(line, "/usr/lib/menu") != 0) // We have a (global) debian menu
			{
				if (parseDebianFile(line, 1, shortname, size, timestamp, callback) == 0)
					break;
			}
			if (strstr(line, debianUser) != 0) // We have a (user) debian menu
			{
				if (parseDebianFile(line, 0, shortname, size, timestamp, callback) == 0)
					break;
			}
		}
	}
	
	free(fileList);
}

void autopackageSearch(void (*callback)(struct ApplicationInfo*))
{
	/* This one-liner lists the installed packages in the format:
	   path|size|timestamp|shortname */
	char* packages = runCommand("bash -c \"function outp { _loadPackageEnvironment \\\"\\$1\\\"; echo \\\"\\$1|\\$EXPANDSIZE|\\$TIMESTAMPINSTALL|\\$SHORTNAME\\\"; }; [ -e /etc/autopackage/config ] && source /etc/autopackage/config; [ -e ~/.config/autopackage/config ] && source ~/.config/autopackage/config; source \\\"\\$autopackage_prefix/share/autopackage/apkg-funclib\\\"; _initializeAutopackage; l=\\$(_listPackagesDB); for p in \\$l; do outp \\\"\\$p\\\"; done\"");
	
	int len = strlen(packages);
	int lastMarker = 0;
	char* path = NULL;
	char* size = NULL;
	char* timestamp = NULL;
	char* shortname = NULL;
	int i;
	for (i=0 ; i<=len ; i++)
	{
		if ((packages[i] == '|') && (path == 0))
		{
			path = (char*) malloc(i - lastMarker + 1);
			memcpy(path, packages + lastMarker, i - lastMarker);
			path[i - lastMarker] = '\0';
			lastMarker = i + 1;
		}
		else if ((packages[i] == '|') && (size == 0))
		{
			size = (char*) malloc(i - lastMarker + 1);
			memcpy(size, packages + lastMarker, i - lastMarker);
			size[i - lastMarker] = '\0';
			lastMarker = i + 1;
		}
		else if ((packages[i] == '|') && (timestamp == 0))
		{
			timestamp = (char*) malloc(i - lastMarker + 1);
			memcpy(timestamp, packages + lastMarker, i - lastMarker);
			timestamp[i - lastMarker] = '\0';
			lastMarker = i + 1;
		}
		else if ((packages[i] == '\n') && (shortname == NULL))
		{
			shortname = (char*) malloc(i - lastMarker + 1);
			memcpy(shortname, packages + lastMarker, i - lastMarker);
			shortname[i - lastMarker] = '\0';
			lastMarker = i + 1;
			readFileList(path, shortname, atoi(size), timestamp, callback);
			
			free(path); path = NULL;
			free(shortname); shortname = NULL;
			free(size); size = NULL;
			free(timestamp); timestamp = NULL;
		}
	}
	free(packages);
}

void autopackageCleanup()
{

}

#endif

